#  Copyright (C) 2024
#  ABM JSC, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Albakov Ruslan <r.albakov@abm-jsc.ru>
from .email_clients.amazon_ses_email_client.amazon_ses_email_client import AmazonSESEmailClient
from .email_clients.http_email_client.http_email_client import HTTPEmailClient
from .email_clients.smtp_email_client.smtp_email_client import SMTPEmailClient
from .email_sender import EmailSender
from .sending_result import SendingResult

__all__ = (AmazonSESEmailClient, HTTPEmailClient, SMTPEmailClient, EmailSender, SendingResult)
